// ----------------------------------------------------------------------------------------
//
//  demos/cfax_lcd/main.cpp
//    
//  Bob Cook Development, Robotics Library
//  http://www.bobcookdev.com/rl/
//    
//  This file demonstrates the SparkFun CFAX Graphic LCD with an ATMega88.
//
//  Copyright (C) 2007 Bob Cook
//
//      This program is free software; you can redistribute it and/or modify it
//      under the terms of the GNU General Public License as published by the
//      Free Software Foundation; either version 2 of the License, or (at your
//      option) any later version.
//    
//      This program is distributed in the hope that it will be useful, but 
//      WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
//      or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
//      for more details.
//    
//      You should have received a copy of the GNU General Public License along
//      with this program; if not, write to the Free Software Foundation, Inc., 
//      51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//
// ----------------------------------------------------------------------------------------


#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include <stdio.h>

#include "packages/avr/device/adc.h"
#include "packages/avr/device/spinwait.h"
#include "packages/avr/lcd/cfax/cfax.h"


// ----------------------------------------------------------------------------------------

inline void led_off()
{
    PORTC |= ( 1 << PC5 );
}


inline void led_on()
{
    PORTC &= ~( 1 << PC5 );
}


// ----------------------------------------------------------------------------------------

static void demo_1()
{
    cfax_write_pgm( PSTR("Hello World!"), 0, 0 );
    cfax_write_pgm( PSTR("CFAX 128 x 64 LCD"), 0, 1 );
    cfax_write_pgm( PSTR("Neat Stuff from SFE"), 0, 3 );
    
    cfax_rect( 10, 50, 10, 7 );
    cfax_fill( 23, 50, 10, 7 );
    cfax_fill( 10, 40, 10, 7 );

    cfax_line( 25, 40, 60, 60, true );
    cfax_line( 25, 40, 70, 60, true );
    cfax_line( 25, 40, 80, 60, true );
    cfax_line( 25, 40, 90, 60, true );

    uint8_t y0  = 40;
    int8_t  dir = 1;

    for ( ;; )
    {
        uint8_t y1 = 100 - y0;

        cfax_line( 100, y0, 120, y1, true );
        spinwait_delay_ms( 250 );
        cfax_line( 100, y0, 120, y1, false );

        y0 += dir;

        if ( y0 > 59 )
        {
            dir = -1;
        }
        else if ( y0 < 41 )
        {
            dir = 1;
        }

        uint8_t touch_x;
        uint8_t touch_y;

        if ( cfax_touchscreen( &touch_x, &touch_y ) )
        {
            break;
        }
    }
}


// ----------------------------------------------------------------------------------------

static const prog_uint8_t glcdimage_pix[] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xfe, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x01, 0xe0, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x38, 0x00, 0x00, 0x38, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x00, 0x00, 0x06, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x01, 0x80, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x00, 0x00, 0x00, 0x00, 0x30, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00, 0x00, 0x18, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 
    0x00, 0x9f, 0xf1, 0xf8, 0xfc, 0x7e, 0x3f, 0x80, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 
    0x01, 0xbf, 0xfb, 0xfd, 0xfe, 0xff, 0x7f, 0x00, 0x07, 0xe0, 0x00, 0x1f, 0xe0, 0x00, 0x1f, 0xe0, 
    0x01, 0xb6, 0xdb, 0x0d, 0x86, 0xc3, 0x00, 0x00, 0x07, 0xf0, 0x00, 0x1f, 0xa0, 0x00, 0x1f, 0xe0, 
    0x01, 0xb6, 0xdb, 0x0d, 0x86, 0xcf, 0x7f, 0x00, 0x0d, 0xdc, 0x00, 0x16, 0xf0, 0x00, 0x7f, 0x60, 
    0x01, 0xb6, 0xdb, 0x0d, 0x86, 0xde, 0x3f, 0x80, 0x0c, 0xff, 0x80, 0x37, 0xd0, 0x07, 0xde, 0x20, 
    0x01, 0xb6, 0xdb, 0x0d, 0x86, 0xc0, 0x01, 0x80, 0x1c, 0x78, 0xe0, 0x35, 0x50, 0x0c, 0x3c, 0x30, 
    0x01, 0xb6, 0xdb, 0xfd, 0x9e, 0xfe, 0x3f, 0x80, 0x14, 0x0e, 0x3c, 0x37, 0xf0, 0x78, 0xe0, 0x30, 
    0x01, 0x24, 0x49, 0xfd, 0xbc, 0x7f, 0x7f, 0x00, 0x24, 0x03, 0x8f, 0xf7, 0xaf, 0xe3, 0x80, 0x28, 
    0x00, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x00, 0x25, 0x80, 0xf9, 0xff, 0xfe, 0x3e, 0x03, 0xa8, 
    0x00, 0x00, 0x00, 0x01, 0xfe, 0x00, 0x00, 0x00, 0x47, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe4, 
    0x00, 0x00, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00, 0x47, 0xff, 0xff, 0x8c, 0xf3, 0xff, 0xff, 0xe4, 
    0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0xc7, 0xf0, 0x18, 0xc0, 0x06, 0x30, 0x1f, 0xe6, 
    0x00, 0x00, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00, 0x85, 0xff, 0xff, 0xe0, 0x0f, 0xff, 0xff, 0x22, 
    0x00, 0x00, 0x00, 0x01, 0xfe, 0x00, 0x00, 0x00, 0x84, 0x00, 0x00, 0xe0, 0x9c, 0x00, 0x00, 0x22, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x84, 0x00, 0x00, 0x7e, 0xf8, 0x00, 0x00, 0x23, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x04, 0x00, 0x00, 0x20, 0x08, 0x00, 0x00, 0x21, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x04, 0x00, 0x00, 0x20, 0x18, 0x00, 0x00, 0x21, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x04, 0x00, 0x00, 0x30, 0x10, 0x00, 0x00, 0x21, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x04, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x21, 
    0x00, 0x3f, 0xf0, 0x03, 0xff, 0x00, 0x3f, 0xc1, 0x04, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x21, 
    0x00, 0x7f, 0xe0, 0x07, 0xfe, 0x00, 0x7f, 0xc1, 0x04, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x21, 
    0x00, 0xff, 0xc0, 0x0f, 0xfc, 0x00, 0xff, 0xc1, 0x04, 0x00, 0x00, 0x11, 0x10, 0x00, 0x00, 0x21, 
    0x01, 0xff, 0x80, 0x1f, 0xf8, 0x01, 0xff, 0x81, 0x04, 0x00, 0x00, 0x18, 0x10, 0x00, 0x00, 0x21, 
    0x03, 0xff, 0x00, 0x3f, 0xf0, 0x03, 0xff, 0x01, 0x04, 0x00, 0x00, 0x3c, 0x30, 0x00, 0x00, 0x21, 
    0x03, 0xfe, 0x00, 0x7f, 0xe0, 0x07, 0xfe, 0x01, 0x04, 0x00, 0x00, 0x20, 0x10, 0x00, 0x00, 0x21, 
    0x03, 0xfc, 0x00, 0xff, 0xc0, 0x0f, 0xfc, 0x01, 0x04, 0x00, 0x00, 0x60, 0x18, 0x00, 0x00, 0x21, 
    0x03, 0xf8, 0x01, 0xff, 0x80, 0x1f, 0xf8, 0x01, 0x04, 0x00, 0x00, 0x60, 0x18, 0x00, 0x00, 0x21, 
    0x03, 0xf0, 0x03, 0xff, 0x00, 0x3f, 0xf0, 0x01, 0x04, 0x00, 0x00, 0xe1, 0x1c, 0x00, 0x00, 0x21, 
    0x03, 0xe0, 0x07, 0xfe, 0x00, 0x7f, 0xe0, 0x01, 0x04, 0x00, 0x00, 0xa3, 0x94, 0x00, 0x00, 0x21, 
    0x03, 0xc0, 0x0f, 0xfc, 0x00, 0xff, 0xc0, 0x01, 0x84, 0x00, 0x00, 0xa7, 0x96, 0x00, 0x00, 0x23, 
    0x03, 0x80, 0x1f, 0xf8, 0x01, 0xff, 0x80, 0x00, 0x84, 0x00, 0x01, 0xad, 0xd2, 0x00, 0x00, 0x22, 
    0x03, 0x00, 0x3f, 0xf0, 0x03, 0xff, 0x00, 0x00, 0x84, 0x00, 0x01, 0x3b, 0x52, 0x00, 0x00, 0x22, 
    0x02, 0x00, 0x7f, 0xe0, 0x07, 0xfe, 0x00, 0x40, 0xc4, 0x00, 0x01, 0x33, 0x73, 0x00, 0x00, 0x26, 
    0x00, 0x00, 0xff, 0xc0, 0x0f, 0xfc, 0x00, 0xc0, 0x44, 0x00, 0x03, 0x33, 0x39, 0x00, 0x00, 0x24, 
    0x00, 0x01, 0xff, 0x80, 0x1f, 0xf8, 0x01, 0xc0, 0x64, 0x00, 0x06, 0x23, 0x19, 0x00, 0x00, 0x2c, 
    0x00, 0x03, 0xff, 0x00, 0x3f, 0xf0, 0x03, 0xc0, 0x24, 0x00, 0x04, 0x73, 0x9d, 0x80, 0x00, 0x28, 
    0x00, 0x07, 0xfe, 0x00, 0x7f, 0xe0, 0x07, 0xc0, 0x34, 0x00, 0x0e, 0xdb, 0x9c, 0x80, 0x00, 0x38, 
    0x00, 0x0f, 0xfc, 0x00, 0xff, 0xc0, 0x0f, 0xc0, 0x14, 0x00, 0x1b, 0xbb, 0xbf, 0x80, 0x00, 0x30, 
    0x00, 0x1f, 0xf8, 0x01, 0xff, 0x80, 0x1f, 0xc0, 0x1c, 0x00, 0x33, 0x2b, 0xa4, 0xc0, 0x00, 0x30, 
    0x00, 0x3f, 0xf0, 0x03, 0xff, 0x00, 0x3f, 0xc0, 0x0c, 0x00, 0x22, 0x2f, 0x66, 0x60, 0x00, 0x60, 
    0x00, 0x7f, 0xe0, 0x07, 0xfe, 0x00, 0x7f, 0xc0, 0x0c, 0x00, 0x66, 0x26, 0x62, 0x20, 0x00, 0x60, 
    0x00, 0xff, 0xc0, 0x0f, 0xfc, 0x00, 0xff, 0xc0, 0x06, 0x00, 0x44, 0x26, 0x22, 0x30, 0x00, 0xe0, 
    0x01, 0xff, 0x80, 0x1f, 0xf8, 0x01, 0xff, 0x80, 0x07, 0x00, 0x4c, 0x26, 0x23, 0x10, 0x01, 0xa0, 
    0x03, 0xff, 0x00, 0x3f, 0xf0, 0x03, 0xff, 0x00, 0x05, 0x80, 0xc8, 0x26, 0x21, 0x90, 0x03, 0x20, 
    0x03, 0xfe, 0x00, 0x7f, 0xe0, 0x07, 0xfe, 0x00, 0x04, 0xc0, 0x98, 0x26, 0x20, 0x98, 0x06, 0x20, 
    0x03, 0xfc, 0x00, 0xff, 0xc0, 0x0f, 0xfc, 0x00, 0x04, 0x60, 0xb0, 0x27, 0x20, 0xc8, 0x0c, 0x20, 
    0x03, 0xf8, 0x01, 0xff, 0x80, 0x1f, 0xf8, 0x00, 0x04, 0x30, 0xa0, 0x37, 0x20, 0x68, 0x18, 0x20, 
    0x03, 0xf0, 0x03, 0xff, 0x00, 0x3f, 0xf0, 0x00, 0x04, 0x0f, 0xe0, 0x17, 0x20, 0x3f, 0xe0, 0x20, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x07, 0x60, 0x13, 0x20, 0x10, 0xc0, 0x20, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x01, 0xe0, 0x1b, 0xa0, 0x13, 0x00, 0x20, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0xe0, 0x0a, 0xa0, 0x1c, 0x00, 0x20, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x1c, 0x09, 0x98, 0x30, 0x00, 0x20, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x07, 0x89, 0x8c, 0x60, 0x00, 0x20, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 
};

static void demo_2()
{
    cfax_bitmap( 0, 0, 128, 64, glcdimage_pix );

    for ( ;; )
    {
        spinwait_delay_ms( 100 );

        uint8_t touch_x;
        uint8_t touch_y;

        if ( cfax_touchscreen( &touch_x, &touch_y ) )
        {
            break;
        }
    }
}


// ----------------------------------------------------------------------------------------

static void demo_3()
{
    cfax_write_pgm( PSTR("Built-in Icons"), 0, 1 );
    
    uint8_t icon = 0;
    bool    state = true;

    for ( ;; )
    {
        cfax_set_icon( static_cast< cfax_icon_t >( icon++ ), state );

        spinwait_delay_ms( 125 );

        if ( icon > CFAX_NUM_ICONS )
        {
            state = ! state;
            icon = 0;
        }

        uint8_t touch_x;
        uint8_t touch_y;

        if ( cfax_touchscreen( &touch_x, &touch_y ) )
        {
            break;
        }
    }
}


// ----------------------------------------------------------------------------------------

static void wait_for_no_touch()
{
    for ( ;; )
    {
        spinwait_delay_ms( 100 );

        uint8_t touch_x;
        uint8_t touch_y;

        if ( ! cfax_touchscreen( &touch_x, &touch_y ) )
        {
            break;
        }
    }
}


// ----------------------------------------------------------------------------------------

int main()
{
    cli();

    TIMSK0 = 0;
    TIMSK1 = 0;
    TIMSK2 = 0;

    sei();

    //--    Status LED: PORTC.5

    DDRC |= ( 1 << PC5 );
    led_off();

    adc_init();
    cfax_init();

    //--    Do the demonstration(s).

    for ( ;; )
    {
        led_on();
        spinwait_delay_ms( 125 );
        led_off();

        demo_1();

        cfax_clear_screen();
        cfax_clear_icons();

        wait_for_no_touch();

        led_on();
        spinwait_delay_ms( 125 );
        led_off();

        demo_2();

        cfax_clear_screen();
        cfax_clear_icons();

        wait_for_no_touch();

        led_on();
        spinwait_delay_ms( 125 );
        led_off();

        demo_3();

        cfax_clear_screen();
        cfax_clear_icons();

        wait_for_no_touch();

    }

    return 0;
}


// ----------------------------------------------------------------------------------------
//  end of main.cpp
// ----------------------------------------------------------------------------------------

