# -----------------------------------------------------------------------------------------
#
#   jamrules.unix.inc
#
#   Bob Cook Development, Robotics Library
#   http://www.bobcookdev.com/rl/
#    
#   Implements the jamrules actions for a generic Unix-ish platform.
#
#   Copyright (C) 2009 Bob Cook
#
#       This program is free software; you can redistribute it and/or modify it
#       under the terms of the GNU General Public License as published by the
#       Free Software Foundation; either version 2 of the License, or (at your
#       option) any later version.
#     
#       This program is distributed in the hope that it will be useful, but 
#       WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
#       or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
#       for more details.
#     
#       You should have received a copy of the GNU General Public License along
#       with this program; if not, write to the Free Software Foundation, Inc., 
#       51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# -----------------------------------------------------------------------------------------


if $(TRACE) { echo "trace /jamrules.unix.inc" ; }


# -----------------------------------------------------------------------------------------
# ----- avr (8-bit) support ---------------------------------------------------------------

# -----------------------------------------------------------------------------------------

actions avr_as
{
    echo $(2)
    avr-gcc -mmcu=$(AVR_DEVICE) -c -o $(1) -I$(AVR_INCDIR) $(2)
}


# -----------------------------------------------------------------------------------------

actions avr_cc
{
    echo $(2)
    avr-gcc -mmcu=$(AVR_DEVICE) -Wall -Os -mcall-prologues -c -o $(1) -I$(AVR_INCDIR) $(2)
}


# -----------------------------------------------------------------------------------------

actions avr_c++
{
    echo $(2)
    avr-gcc -mmcu=$(AVR_DEVICE) -Wall -Os -mcall-prologues -c -o $(1) -I$(AVR_INCDIR) $(2)
}


# -----------------------------------------------------------------------------------------

actions avr_link
{
    echo $(1)
    avr-gcc -g -Wall $(AVR_OPTIM) -mmcu=$(AVR_DEVICE) -o $(1) -L$(AVR_LIBDIR) -l$(AVR_LIBS) $(AVR_EXTRA_LIBS) -Wl,-Map,$(AVR_LINKMAP) $(2)
}


# -----------------------------------------------------------------------------------------

actions avr_list_from_executable
{
    echo $(1)
    avr-objdump -h -S $(2) > $(1)
}


# -----------------------------------------------------------------------------------------

actions avr_texthex_from_executable
{
    echo $(1)
    avr-objcopy -j .text -j .data -O ihex $(2) $(1)
}


# -----------------------------------------------------------------------------------------

actions avr_eepromhex_from_executable
{
    echo $(1)
    avr-objcopy -j .eeprom --change-section-lma .eeprom=0 -O ihex $(2) $(1)
}


# -----------------------------------------------------------------------------------------
# ----- hammer (samsung s3c2410a arm920t) support -----------------------------------------

HAMMER_TOOLS_PATH = "/home/bob/software/hammer/buildroot-2009.05/build_arm/staging_dir" ;

# -----------------------------------------------------------------------------------------

actions hammer_as
{
    echo $(2)
    $(HAMMER_TOOLS_PATH)/usr/bin/arm-linux-uclibc-gcc -c -o $(1) -I$(HAMMER_INCDIR) $(2)
}


# -----------------------------------------------------------------------------------------

actions hammer_cc
{
    echo $(2)
    $(HAMMER_TOOLS_PATH)/usr/bin/arm-linux-uclibc-gcc -Wall $(HAMMER_OPTIM) -c -o $(1) -I$(HAMMER_INCDIR) $(2)
}


# -----------------------------------------------------------------------------------------

actions hammer_c++
{
    echo $(2)
    $(HAMMER_TOOLS_PATH)/usr/bin/arm-linux-uclibc-g++ -Wall $(HAMMER_OPTIM) -c -o $(1) -I$(HAMMER_INCDIR) $(2)
}


# -----------------------------------------------------------------------------------------
# avr-gcc -g -Wall $(AVR_OPTIM) -mmcu=$(AVR_DEVICE) -o $(1) -L$(AVR_LIBDIR) -l$(AVR_LIBS) $(AVR_EXTRA_LIBS) -Wl,-Map,$(AVR_LINKMAP) $(2)

actions hammer_link
{
    echo $(1)
    $(HAMMER_TOOLS_PATH)/usr/bin/arm-linux-uclibc-g++ -o $(1) -L$(HAMMER_LIBDIR) -l$(HAMMER_LIBS) -Wl,-Map,$(HAMMER_LINKMAP) $(2)
}


# -----------------------------------------------------------------------------------------

actions hammer_list_from_executable
{
    echo $(1)
    $(HAMMER_TOOLS_PATH)/usr/bin/arm-linux-uclibc-objdump -h -S $(2) > $(1)
}

