// ----------------------------------------------------------------------------------------
//
//  common/util/printutils.h
//    
//  Bob Cook Development, Robotics Library
//  http://www.bobcookdev.com/rl/
//    
//  This file provides printf-helper functions and macros.
//
//  Copyright (C) 2007 Bob Cook
//
//      This program is free software; you can redistribute it and/or modify it
//      under the terms of the GNU General Public License as published by the
//      Free Software Foundation; either version 2 of the License, or (at your
//      option) any later version.
//    
//      This program is distributed in the hope that it will be useful, but 
//      WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
//      or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
//      for more details.
//    
//      You should have received a copy of the GNU General Public License along
//      with this program; if not, write to the Free Software Foundation, Inc., 
//      51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//
// ----------------------------------------------------------------------------------------


#include "printutils.h"

#include <stdio.h>


// ----------------------------------------------------------------------------------------

void print_hex( uint8_t byte )
{
    uint8_t high_nybble = byte >> 4;

    if ( high_nybble < 0x0a )
    {
        putchar( '0' + high_nybble );
    }
    else
    {
        putchar( 'A' + ( high_nybble - 0x0a ) );
    }
    
    byte &= 0x0f;

    if ( byte < 0x0a )
    {
        putchar( '0' + byte );
    }
    else
    {
        putchar( 'A' + ( byte - 0x0a ) );
    }
}


// ----------------------------------------------------------------------------------------

void print_hex( const uint16_t& word )
{
    uint8_t high_byte = static_cast< uint8_t >( word >> 8 );
    print_hex( high_byte );
    print_hex( static_cast< uint8_t >( word ) );
}


// ----------------------------------------------------------------------------------------

void print_hex( const uint32_t& longword )
{
    uint16_t high_word = static_cast< uint16_t >( longword >> 16 );
    print_hex( high_word );
    print_hex( static_cast< uint16_t >( longword ) );
}


// ----------------------------------------------------------------------------------------

void print_bits( uint8_t byte )
{
    for ( uint8_t i = 0; i < 8; i++ )
    {
        if ( byte & 0x80 )
        {
            putchar( '1' );
        }
        else
        {
            putchar( '0' );
        }
        byte <<= 1;
    }
}


// ----------------------------------------------------------------------------------------

void print_bits( uint16_t word )
{
    for ( uint8_t i = 0; i < 16; i++ )
    {
        if ( word & 0x8000 )
        {
            putchar( '1' );
        }
        else
        {
            putchar( '0' );
        }
        word <<= 1;
    }
}


// ----------------------------------------------------------------------------------------

void print_bits( uint32_t longword )
{
    for ( uint8_t i = 0; i < 32; i++ )
    {
        if ( longword & 0x80000000 )
        {
            putchar( '1' );
        }
        else
        {
            putchar( '0' );
        }
        longword <<= 1;
    }
}


// ----------------------------------------------------------------------------------------
//  end of printutils.cpp
// ----------------------------------------------------------------------------------------

